﻿SpeakinSpell = LibStub("AceAddon-3.0"):GetAddon("SpeakinSpell")

-- load localized text including default speeches
local L					= LibStub("AceLocale-3.0"):GetLocale("SpeakinSpell",					false)
local DEFAULT_SPEECHES	= LibStub("AceLocale-3.0"):GetLocale("SpeakinSpell_DEFAULT_SPEECHES",	false)


-------------------------------------------------------------------------------
-- GENERAL INITIALIZATION AND MAIN ENTRY POINTS
-------------------------------------------------------------------------------


-- our saved data is stored in SpeakinSpellSavedData.EventTable
-- we must declare the parent table SpeakinSpellSavedData
-- but we know whether real data has been loaded or not if SpeakinSpellSavedData.EventTable exists
-- NOTE: SpeakinSpellSavedData.EventTable may be an empty table
SpeakinSpellSavedData = {}

function SpeakinSpell:InitDefaultSavedData()
	-- copy defaults into current settings using class-specific defaults
	local playerClass, englishClass = UnitClass("player"); --englishClass is the UPPERCASE class name in English
	
	SpeakinSpellSavedData.Version = SpeakinSpell.CURRENT_VERSION
	SpeakinSpellSavedData.ShowSpellEvents = false
	SpeakinSpellSavedData.ShowDebugMessages = false
	SpeakinSpellSavedData.EnableAllMessages = true
	
	SpeakinSpellSavedData.EventTable = nil
	SpeakinSpellSavedData.EventTable = {}
	SpeakinSpellSavedData.EventTable = self:CopyTable( DEFAULT_SPEECHES.CLASSES[englishClass] )
	
	SpeakinSpellSavedData.NewEventsDetected = nil
	SpeakinSpellSavedData.NewEventsDetected = {}

	-- remove faction-specific spells and reset runtime data such as localizable display names
	for key, EventTableEntry in pairs(SpeakinSpellSavedData.EventTable) do
		-- rebuild some info that's normally built dynamically
		local de = EventTableEntry.DetectedEvent
		de.DisplayName = self:FormatDisplayName(de)
		de.key = key
		de.SpellLink = self:SafeGetSpellLink(de)
		
		if EventTableEntry.Faction ~= nil then
			if EventTableEntry.Faction ~= UnitFactionGroup("player") then
				SpeakinSpellSavedData.EventTable[key] = nil
			end
		end
	end
end


function SpeakinSpell:ResetToDefaults(FrameName) --TODOFUTURE: reset only the current frame's defaults
	SpeakinSpellSavedData.EventTable = nil
	
	-- copy defaults into current settings using class-specific defaults
	self:InitDefaultSavedData()
	
	-- update/reset runtime data state
	self:InitRuntimeData()
	self:CurrentMessagesGUI_Refresh() -- this might be redundant but doesn't seem to hurt
	
	-- if the options GUI is showing right now, we need to refresh it to make it show the new data
	self:RefreshFrame("General")
	self:RefreshFrame("CreateNew")
	self:RefreshFrame("CurrentMessagesGUI")
	
	self:Print(L["Default settings restored"])
end


function SpeakinSpell:InitRuntimeData()
	-- this is just a declaration
	self.RuntimeData = {
		LastMessage = {},			-- just an empty table for now, this will be [spellname] = "last random speech triggered by this spell" to prevent repeats
	}

	self:InitOptionsGUIState() -- sets up event lists and current selections
end


function SpeakinSpell:OnInitialize()
    -- Called when the addon is loaded

	self:RegisterAllEvents()
	self:RegisterChatCommands()	-- register chat commands
	-- the rest of the initialization does not occur until VARIABLES_LOADED
	
	-- done loading
    self:Print( string.format(L["Loaded version %s"], self.CURRENT_VERSION) )
end


function SpeakinSpell:OnEnable()
	-- Called when the addon is enabled
	-- self:RegisterAllEvents() -- did this OnInitialize
	self:CurrentMessagesGUI_Refresh() -- make sure the options GUI is setup correctly
end


function SpeakinSpell:OnDisable()
    -- Called when the addon is disabled
	-- we don't need to do anything here
end

